/**
 *	@file	GetPixelsAppUi.cpp
 *	@author	ENDO
 */

#include <e32base.h>
#include <s32file.h>
#include <imageconversion.h>	// for ICL.
#include <sacls.h>				// for KUidChargerStatus.

#include <avkon.hrh>
#include <stringloader.h>
#include <aknviewappui.h>	// for CAknViewAppUi.
#include <aknglobalnote.h>	// for CAknGlobalNote.
#include <vibractrl.h>		// for CVibraControl.
#include <pathinfo.h>		// for PathInfo.

#include <GetPixels.rsg>
#include "GetPixels.hrh"
#include "GetPixelsDocument.h"
#include "GetPixelsAppUi.h"
#include "GetPixelsMainView.h"
#include "GetPixelsSettingsView.h"
#include "GetPixelsImageSettingsView.h"
#include "GetPixelsSettings.h"
#include "GetPixelsUtilities.h"

#include "ScreenCaptor.h"
#include "ImageSaver.h"
#include "KeyEventWatcher.h"
#include "SystemEventWatcher.h"
#include "MyAssert.h"

LOCAL_D const TInt KMilliSecondsVibraCompletion = 1000;
LOCAL_D const TInt KMilliSecondsVibraNotification = 100;

CGetPixelsAppUi::CGetPixelsAppUi(void) :
	iEnv(CEikonEnv::Static())
	{
	}

void CGetPixelsAppUi::ConstructL(void)
	{
	BaseConstructL(EAknEnableSkin);

	iScreenCaptor = CScreenCaptor::NewL(*iEnv->ScreenDevice());
	iImageSaver = CImageSaver::NewL(iEnv->FsSession(), *this);
	iKeyEventWatcher = CKeyEventWatcher::NewL(iEnv->RootWin(), this);
	iSystemEventWatcher = CSystemEventWatcher::NewL(*this);

	iPeriodicTimer = CPeriodic::NewL(CPeriodic::EPriorityStandard);
	iGlobalNote = CAknGlobalNote::NewL();
	iVibraController = VibraFactory::NewL();

	iMainView = CGetPixelsMainView::NewL();
	AddViewL(iMainView);
	iSettingsView = CGetPixelsSettingsView::NewL();
	AddViewL(iSettingsView);
	iImageSettingsView = CGetPixelsImageSettingsView::NewL();
	AddViewL(iImageSettingsView);

	SetDefaultViewL(*iMainView);
	}

CGetPixelsAppUi::~CGetPixelsAppUi()
	{
	iViewHistoryStack.Close();

	delete iVibraController;
	delete iGlobalNote;
	delete iPeriodicTimer;
	delete iSystemEventWatcher;
	delete iKeyEventWatcher;
	delete iImageSaver;
	delete iScreenCaptor;
	}

void CGetPixelsAppUi::HandleCommandL(TInt aCommand)
	{
	switch (aCommand)
		{
	case EEikCmdExit:
	case EAknSoftkeyExit:
		SaveL();
		Exit();
		break;

	case ECommandCapture:
		MY_ASSERT(GetModel() != NULL);
		if (GetModel()->DelayTime() == 0)
			{
			CommandCaptureL();
			}
		else
			{
			CommandTriggerTimerL();
			}
		break;

	case ECommandSwitchToSettingsView:
			{
			const TUid viewId = {EGetPixelsSettingsViewId};
			ActivateLocalViewL(viewId);

			iViewHistoryStack.Append(iView->ViewId().iViewUid);
			}
		break;

	case ECommandSwitchToImageSettingsView:
			{
			const TUid viewId = {EGetPixelsImageSettingsViewId};
			ActivateLocalViewL(viewId);

			iViewHistoryStack.Append(iView->ViewId().iViewUid);
			}
		break;

	case ECommandBackToPreviousView:
			{
			MY_ASSERT(iViewHistoryStack.Count() > 0);

			const TInt index = iViewHistoryStack.Count() - 1;
			ActivateLocalViewL(iViewHistoryStack[index]);
			iViewHistoryStack.Remove(index);
			}
		break;

	default:
		MY_ASSERT(EFalse);
		break;
		}
	}

TKeyResponse CGetPixelsAppUi::HandleKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType)
	{
	if (aType == EEventKeyDown)
		{
		MY_ASSERT(GetModel() != NULL);
		if (static_cast<TUint>(aKeyEvent.iScanCode) == GetModel()->TriggerKeyScanCode() && (aKeyEvent.iModifiers & EModifierShift) != 0U)
			{
			HandleCommandL(ECommandCapture);
			return EKeyWasConsumed;
			}
		}

	return EKeyWasNotConsumed;
	}

void CGetPixelsAppUi::HandleModelChangeL(void)
	{
	MY_ASSERT(GetModel() != NULL);
	RestartWatchingKeyEventL(GetModel()->TriggerKeyScanCode());
	RestartWatchingSystemEventL(GetModel()->TriggerSystemEvent());
	}

void CGetPixelsAppUi::NotifyCompletion(TInt aStatus)
	{
	MY_ASSERT(GetModel() != NULL);
	if (aStatus == KErrNone)
		{
		if (GetModel()->EnableConfirmationNote())
			{
			MY_ASSERT(iGlobalNote != NULL);
			iGlobalNote->ShowNoteL(EAknGlobalConfirmationNote, CapturedImageFilePath());
			}
		if (GetModel()->EnableVibrationNotification())
			{
			MY_ASSERT(iVibraController != NULL);
			iVibraController->StartVibraL(KMilliSecondsVibraCompletion);
			}
		}
	else
		{
		if (GetModel()->EnableConfirmationNote())
			{
			HBufC* noteText = NULL;
			TRAPD(ret, noteText = StringLoader::LoadL(R_GETPIXELS_TEXT_ENCODE_ERROR));
			if (ret == KErrNone)
				{
				MY_ASSERT(iGlobalNote != NULL);
				iGlobalNote->ShowNoteL(EAknGlobalErrorNote, *noteText);
				delete noteText;
				noteText = NULL;
				}
			}
		iCapturedImageFilePath.Zero();
		}

	MY_ASSERT(iMainView != NULL);
	iMainView->DrawDeferred();
	}

void CGetPixelsAppUi::NotifyGlobalKeyEvent(const TWsEvent& aEvent)
	{
	if (aEvent.Type() == EEventKeyDown)
		{
		const TKeyEvent* const keyEvent = aEvent.Key();

		MY_ASSERT(GetModel() != NULL);
		if (static_cast<TUint>(keyEvent->iScanCode) == GetModel()->TriggerKeyScanCode() && (keyEvent->iModifiers & EModifierShift) != 0U)
			{
			HandleCommandL(ECommandCapture);
			}
		}
	}

void CGetPixelsAppUi::NotifySystemEvent(const TSysAgentEvent& aSysAgentEvent)
	{
	TSysAgentEvent sysAgentEvent = aSysAgentEvent;
	if (sysAgentEvent.Uid() == KUidChargerStatus && sysAgentEvent.State() == ESAChargerConnected)
		{
		HandleCommandL(ECommandCapture);
		}
	}

void CGetPixelsAppUi::RestartWatchingKeyEventL(TUint aScanCode)
	{
	iKeyEventWatcher->Cancel();
	iKeyEventWatcher->StartCaptureKeyUpAndDownsL(aScanCode, EModifierShift, EModifierShift);
	}

void CGetPixelsAppUi::RestartWatchingSystemEventL(TUid aUid)
	{
	iSystemEventWatcher->Cancel();
	if (aUid != KNullUid)
		{
		iSystemEventWatcher->StartL(aUid);
		}
	}

CGetPixelsSettings* CGetPixelsAppUi::GetModel(void) const
	{
	CGetPixelsDocument* document = static_cast<CGetPixelsDocument*>(Document());
	if (document == NULL)
		{
		return NULL;
		}

	return document->Model();
	}

const CFbsBitmap* CGetPixelsAppUi::CapturedBitmap(void) const
	{
	if (iScreenCaptor == NULL)
		{
		return NULL;
		}

	return iScreenCaptor->CapturedBitmap();
	}

void CGetPixelsAppUi::CommandCaptureL(void)
	{
	TInt ret;

	if (iImageSaver->IsBusy())
		{
		return;
		}

	const CGetPixelsSettings* const settings = GetModel();
	MY_ASSERT(settings != NULL);

	iCapturedImageFilePath.Zero();

	iScreenCaptor->Capture();

	GetPixelsUtilities::GetSavedFilePathL(iCapturedImageFilePath,
		(!settings->EnableSaveToExternal()) ? PathInfo::PhoneMemoryRootPath() : PathInfo::MemoryCardRootPath(),
		settings->SavedDirectoryName(),
		settings->SavedFileName(),
		GetPixelsUtilities::ExtNameFromUid(settings->ImageFormat()));

	iImageSaver->ResetFrameImageDataL();
	const TUid imageFormat(settings->ImageFormat());
	if (imageFormat == KImageTypePNGUid)
		{
		TPngEncodeData pngEncodeData;
		pngEncodeData.iBitsPerPixel = settings->PngBitsPerPixel();
		pngEncodeData.iColor = ETrue;
		pngEncodeData.iPaletted = (settings->PngBitsPerPixel() <= 8);
		pngEncodeData.iLevel = settings->PngCompressLevel();
		iImageSaver->AppendFrameDataL(pngEncodeData);
		}
	else if (imageFormat == KImageTypeBMPUid)
		{
		TBmpImageData bmpImageData;
		bmpImageData.iBitsPerPixel = settings->BmpBitsPerPixel();
		iImageSaver->AppendImageDataL(bmpImageData);
		}
	else if (imageFormat == KImageTypeMBMUid)
		{
		if (settings->MbmDisplayMode() == ENone)
			{
			TInt ret = const_cast<CFbsBitmap*>(iScreenCaptor->CapturedBitmap())->Save(iCapturedImageFilePath);
			NotifyCompletion(ret);
			return;
			}

		TMbmEncodeData mbmEncodeData;
		mbmEncodeData.iDisplayMode = settings->MbmDisplayMode();
		iImageSaver->AppendFrameDataL(mbmEncodeData);
		}
	else if (imageFormat == KImageTypeJPGUid)
		{
		TJpegImageData jpegImageData;
		jpegImageData.iQualityFactor = settings->JpegQualityFactor();
		jpegImageData.iSampleScheme = settings->JpegSampleScheme();
		iImageSaver->AppendImageDataL(jpegImageData);

		if (settings->JpegComment().Length() > 0)
			{
			TJpegComment jpegComment;
			jpegComment.iComment = HBufC8::NewL(settings->JpegComment().Size());
			CleanupStack::PushL(jpegComment.iComment);

			TPtrC8 ptrJpegComment(reinterpret_cast<const TUint8*>(settings->JpegComment().Ptr()), settings->JpegComment().Size());
			jpegComment.iComment->Des().Copy(ptrJpegComment);

			iImageSaver->AppendImageDataL(jpegComment);
			CleanupStack::Pop(jpegComment.iComment);
			}
		}
	else
		{
		MY_ASSERT(EFalse);
		}

	TRAP(ret, iImageSaver->SaveFileL(*iScreenCaptor->CapturedBitmap(), iCapturedImageFilePath, settings->EnableEncodeThread(), imageFormat));
	if (ret == KErrPathNotFound)
		{
		TParsePtrC parser(iCapturedImageFilePath);
		(void)iEnv->FsSession().MkDirAll(parser.DriveAndPath());

		TRAP(ret, iImageSaver->SaveFileL(*iScreenCaptor->CapturedBitmap(), iCapturedImageFilePath, settings->EnableEncodeThread(), imageFormat));
		}

	if (ret != KErrNone)
		{
		iImageSaver->ResetFrameImageDataL();
		iCapturedImageFilePath.Zero();
		User::Leave(ret);
		}
	}

void CGetPixelsAppUi::CommandTriggerTimerL(void)
	{
	MY_ASSERT(iPeriodicTimer != NULL);
	if (!iPeriodicTimer->IsActive())
		{
		iPeriodicTimer->Start(GetModel()->DelayTime() * 1000, KMaxTInt, TCallBack(CGetPixelsAppUi::PeriodicTimerCallbackHandler, this));

		if (GetModel()->EnableConfirmationNote())
			{
			MY_ASSERT(iGlobalNote != NULL);
			HBufC* noteText = StringLoader::LoadLC(R_GETPIXELS_TEXT_START_TIMER);
			iGlobalNote->ShowNoteL(EAknGlobalConfirmationNote, *noteText);
			CleanupStack::PopAndDestroy(noteText);
			}
		if (GetModel()->EnableVibrationNotification())
			{
			MY_ASSERT(iVibraController != NULL);
			iVibraController->StartVibraL(KMilliSecondsVibraNotification);
			}
		}
	else
		{
		iPeriodicTimer->Cancel();

		if (GetModel()->EnableConfirmationNote())
			{
			MY_ASSERT(iGlobalNote != NULL);
			HBufC* noteText = StringLoader::LoadLC(R_GETPIXELS_TEXT_STOP_TIMER);
			iGlobalNote->ShowNoteL(EAknGlobalConfirmationNote, *noteText);
			CleanupStack::PopAndDestroy(noteText);
			}
		}
	}

TInt CGetPixelsAppUi::PeriodicTimerCallbackHandler(TAny* aPtr)
	{
	CGetPixelsAppUi* const appUi = static_cast<CGetPixelsAppUi*>(aPtr);

	TRAPD(ret, appUi->CommandCaptureL());
	appUi->iPeriodicTimer->Cancel();

	return ret;
	}
